unit CSV;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DbTables, StrUtils;

type
  TCSVProgressEvent  = procedure (Sender : TObject; AProgress: LongInt) of object;

  TCSV = class(TComponent)
  private
   FTable           : TTable;
   FCSVMap,
   FCSVFile,
   FIgnoreStr       : string;
   FSeprator,
   FDelimiter,
   FFieldIndicator  : Char;
   FAutoOpen,
   FEmptyTable      : Boolean;
   FBeforeOpenTable,
   FAfterOpenTable,
   FBeforeCloseTable,
   FAfterCloseTable,
   FBeforeEmptyTable,
   FAfterEmptyTable,
   FBeforeExport,
   FAfterExport,
   FBeforeImport,
   FAfterImport,
   FOnAddRecord     : TNotifyEvent;
   FExportProgress,
   FImportProgress  : TCSVProgressEvent;
  protected
   FFile            : TextFile;
   function CountMapItems:Integer;
   function GetMapItem(ItemIndex:Integer;var AField:Boolean):string;
   function GetCSVRecordItem(ItemIndex:Integer;CSVRecord:string):string;
  published
   //properties
   property Table            : TTable            read FTable            write FTable;
   property CSVMap           : string            read FCSVMap           write FCSVMap;
   property CSVFile          : string            read FCSVFile          write FCSVFile;
   property Seprator         : Char              read FSeprator         write FSeprator;
   property FieldIndicator   : Char              read FFieldIndicator   write FFieldIndicator;
   property AutoOpen         : Boolean           read FAutoOpen         write FAutoOpen         default False;
   property IgnoreString     : string            read FIgnoreStr        write FIgnoreStr;
   property Delimiter        : Char              read FDelimiter        write FDelimiter;
   property EmptyTable       : Boolean           read FEmptyTable       write FEmptyTable       default False;
   //events
   property BeforeOpenTable  : TNotifyEvent      read FBeforeOpenTable  write FBeforeOpenTable;
   property AfterOpenTable   : TNotifyEvent      read FAfterOpenTable   write FAfterOpenTable;
   property BeforeCloseTable : TNotifyEvent      read FBeforeCloseTable write FBeforeCloseTable;
   property AfterCloseTable  : TNotifyEvent      read FAfterCloseTable  write FAfterCloseTable;
   property BeforeEmptyTable : TNotifyEvent      read FBeforeEmptyTable write FBeforeEmptyTable;
   property AfterEmptyTable  : TNotifyEvent      read FAfterEmptyTable  write FAfterEmptyTable;
   property BeforeImport     : TNotifyEvent      read FBeforeImport     write FBeforeImport;
   property AfterImport      : TNotifyEvent      read FAfterImport      write FAfterImport;
   property BeforeExport     : TNotifyEvent      read FBeforeExport     write FBeforeExport;
   property AfterExport      : TNotifyEvent      read FAfterExport      write FAfterExport;
   property ExportProgress   : TCSVProgressEvent read FExportProgress   write FExportProgress;
   property ImportProgress   : TCSVProgressEvent read FImportProgress   write FImportProgress;
   property OnAddRecord      : TNotifyEvent      read FOnAddRecord      write FOnAddRecord;
   //methodes
   procedure ExportToTable;
   procedure ImportFromTable;
  end;

procedure Register;

implementation

//published methodes
procedure TCSV.ExportToTable;
var
 RecordString : string;
 i            : Integer;
 C            : LongInt;
 B,
 D            : Boolean;
begin
 //assign and open CSV file
 AssignFile(FFile,FCSVFile);
 Reset(FFile);
 //empty table if nessecary
 if FEmptyTable then
  begin
   if Assigned(FBeforeEmptyTable) then
    FBeforeEmptyTable(Self); 
   B:=FTable.Exclusive;
   FTable.Exclusive:=True;
   FTable.EmptyTable;
   FTable.Exclusive:=B;
   if Assigned(FAfterEmptyTable) then
    FAfterEmptyTable(Self);
  end;
 //open table if nessecary
 if FAutoOpen then
  begin
   if Assigned(FBeforeOpenTable) then
    FBeforeOpenTable(Self);
   FTable.Open;
   if Assigned(FAfterOpenTable) then
    FAfterOpenTable(Self);
  end;
 //export to table from CSV file
 if Assigned(FBeforeExport) then
  FBeforeExport(Self);
 //set the counter to zero 
 C:=0; 
 while not Eof(FFile) do
  begin
   //read from CSV
   Readln(FFile,RecordString);
   //add new record
   FTable.Append;
   for i:=1 to CountMapItems do
    if Uppercase(GetMapItem(i,D)) <> Uppercase(FIgnoreStr) then
     FTable.FieldByName(GetMapItem(i,D)).AsString:=GetCSVRecordItem(i,RecordString);
   //post record
   FTable.Post;
   if Assigned(FOnAddRecord) then
    FOnAddRecord(Self);
   if Assigned(FExportProgress) then
    FExportProgress(Self, C);
   Inc(C);
  end;
 if Assigned(FAfterExport) then
  FAfterExport(Self);
 //close table if nessecary
 if FAutoOpen then
  begin
   if Assigned(FBeforeCloseTable) then
    FBeforeCloseTable(Self);
   FTable.Close;
   if Assigned(FAfterCloseTable) then
    FAfterCloseTable(Self);
  end;
 //close CSV file 
 CloseFile(FFile); 
end;

procedure TCSV.ImportFromTable;
var
 S,
 D,
 T:string;
 i:Integer;
 C:LongInt;
 B:Boolean;
begin
 //assign and open CSV file
 AssignFile(FFile,FCSVFile);
 //emprty CSV if nessecary
 if FEmptyTable then
  begin
   if Assigned(FBeforeEmptyTable) then
    FBeforeEmptyTable(Self);
   Rewrite(FFile);
   if Assigned(FAfterEmptyTable) then
    FAfterEmptyTable(Self);
  end
 else
  Append(FFile);
 //open table if nessecary
 if FAutoOpen then
  begin
   if Assigned(FBeforeOpenTable) then
    FBeforeOpenTable(Self);
   FTable.Open;
   if Assigned(FAfterOpenTable) then
    FAfterOpenTable(Self);
  end;
 //import from table to CSV file
 if Assigned(FBeforeImport) then
  FBeforeImport(Self);
 C:=0;
 FTable.First;
 while not FTable.Eof do
  begin
   S:='';
   for i:=1 to CountMapItems do
    begin
     D:=GetMapItem(i,B);
     if B then //is it a field definition
      T:=FTable.FieldByName(GetMapItem(i,B)).AsString
     else
      T:=GetMapItem(i,B);
     if Trim(T) = '' then
      T:=' ';
     S:=S + T + FSeprator;
    end;
   //delete last seprator 
   Delete(S,Length(S),1);
   Writeln(FFile,S);
   FTable.Next;
   //call progress event
   if Assigned(FImportProgress) then
    FImportProgress(Self, C);
   Inc(C);
  end;
 if Assigned(FAfterImport) then
  FAfterImport(Self);
 //close table if nessecary
 if FAutoOpen then
  begin
   if Assigned(FBeforeCloseTable) then
    FBeforeCloseTable(Self);
   FTable.Close;
   if Assigned(FAfterCloseTable) then
    FAfterCloseTable(Self);
  end;
 //close CSV file
 CloseFile(FFile);
end;

//internal methodes
function TCSV.CountMapItems:Integer;
begin
 Result:=WordCount(FCSVMap,[FSeprator]);
end;

function TCSV.GetMapItem(ItemIndex:Integer;var AField:Boolean):string;
var
 S:string;
begin
 AField:=True;
 S:=ExtractWord(ItemIndex,FCSVMap,[FSeprator]);
 if (Length(S) >= 1) and (S[1] = FFieldIndicator) then
  Result:=Copy(S,2,Length(S) - 1)
 else
  begin
   AField:=False;
   Result:=S;
  end; 
end;

function TCSV.GetCSVRecordItem(ItemIndex:Integer;CSVRecord:string):string;
var
 S:string;
begin
 S:=ExtractWord(ItemIndex,CSVRecord,[FSeprator]);
 if (Length(S) >= 2) and (S[1] = FDelimiter) and (S[Length(S)] = FDelimiter) then
  Result:=Copy(S,2,Length(S) - 2)
 else
  Result:=S; 
end;

procedure Register;
begin
  RegisterComponents('Other', [TCSV]);
end;

end.
